#include <allegro.h>
#include <stdio.h>
#include "gamedata.h"
#include "tiledata.h"

#define MAXTILES 25

typedef struct MAP
{
int x;
int y;
unsigned char *map;
} MAP;

MAP map;
DATAFILE *tiles;

int mapx;
int mapy;

unsigned char currtile;

void SaveMap(char *fn)
{
FILE *fp;

fp=fopen(fn, "wb");
fwrite(&map.x, sizeof(int), 1, fp);
fwrite(&map.y, sizeof(int), 1, fp);
fwrite(map.map, sizeof(unsigned char), map.x * map.y, fp);
fclose(fp);
}

void LoadMap(char *fn)
{
FILE *fp;
int index;

fp=fopen(fn, "rb");
fread(&map.x, sizeof(int), 1, fp);
fread(&map.y, sizeof(int), 1, fp);
map.map = (unsigned char*)realloc(map.map, map.x * map.y);
fread(map.map, sizeof(unsigned char), map.x * map.y, fp);
fclose(fp);

for(index = 0; index < map.x * map.y; index++)
   if(map.map[index] >= MAXTILES)
      map.map[index] = MAXTILES - 1;
}

int FindGrid(int x, int y)
{
return ((x / 32) + ((y / 32) * map.x));
}

void DrawCurrTile()
{
if(mouse_x > (SCREEN_W / 2))
{
   rect(screen,6,6,39,39,0);
   blit((BITMAP*)tiles[currtile].dat,screen,0,0,7,7,32,32);
}
else
{
   rect(screen, SCREEN_W - 38, 6, SCREEN_W - 5, 39, 0);
   blit((BITMAP*)tiles[currtile].dat,screen,0,0,SCREEN_W - 37,7,32,32);
}
}

void DoUI()
{
if(mouse_x < 10)
   mapx -= 12;
if(mouse_x >= SCREEN_W - 10)
   mapx += 12;
if(mouse_y < 10)
   mapy -= 12;
if(mouse_y >= SCREEN_H - 10)
   mapy += 12;

if(mouse_b)
   map.map[FindGrid(mouse_x + mapx, mouse_y + mapy)] = currtile;

if(keypressed())
   switch(readkey() & 0xff)
   {
   case 'c':
   case 'C':
        currtile ++;
        if (currtile == MAXTILES)
           currtile = 0;
        break;

   case 'x':
   case 'X':
        if(currtile == 0)
            currtile = MAXTILES - 1;
        else
            currtile --;
        break;

   case 's':
   case 'S':
        SaveMap("map1.map");
        break;

   case 'l':
   case 'L':
        LoadMap("map1.map");
        break;
   default:
        break;
   }
}

void CheckBoundaries()
{
if(mapx < 0)
   mapx = 0;
if(mapy < 0)
   mapy = 0;
if(mapx > (map.x * 32) - SCREEN_W)
   mapx = (map.x * 32) - SCREEN_W;
if(mapy > (map.y * 32) - SCREEN_H)
   mapy = (map.y * 32) - SCREEN_H;
}

void DrawMap(int x, int y)
{
int startx;
int starty;
int lx;
int ly;
int tilefind;
int sx;
int sy;

startx = x / 32;
starty = y / 32;

for(lx=0; lx<33; lx++)
   for(ly=0; ly<25; ly++)
   {
      tilefind = (lx + startx) + ((ly + starty) * map.x);
      if (tilefind >= map.x * map.y)
         continue;
      sx = (startx * 32) - x + (lx * 32);
      sy = (starty * 32) - y + (ly * 32);
      blit((BITMAP*)tiles[map.map[tilefind]].dat, screen, 0,0,sx,sy,32,32);
   }
}

void create_map(int x, int y)
{
int index;

map.x = x;
map.y = y;
map.map = (unsigned char*)malloc(x*y);

for(index=0;index<x*y;index++)
   map.map[index]=0;
}

void DoEdit()
{
while(1)
{
   DoUI();
   CheckBoundaries();
   vsync();
   DrawMap(mapx, mapy);
   DrawCurrTile();
   rest(50);
   if(key[KEY_ESC])
      break;
}
}

void main()
{
allegro_init();
install_mouse();
install_timer();
install_keyboard();

create_map(80,50);
set_gfx_mode(GFX_GDI,640,480,0,0);

tiles=load_datafile("game.dat");
set_palette((RGB*)tiles[PAL].dat);
//unload_datafile(tiles);

show_mouse(screen);

tiles=load_datafile("tile.dat");

DoEdit();

unload_datafile(tiles);
}                      
END_OF_MAIN();
